import octofuss
from octofuss.treeextra import ListTree
import re
import string
import time
import datetime as dt
import os, os.path
from gettext import gettext as _
from twisted.internet.threads import deferToThread
mydeferred = deferToThread.__get__

LEASE_FILES = ['/var/lib/dhcp3/dhcpd.leases', '/tmp/dhcpd.leases']

class DhcpLeases(octofuss.Tree):
    def __init__(self, name = None):
        super(DhcpLeases, self).__init__(name, doc="DHCP Leases")

        self.leases = {}
        self.load_conf()


    def load_conf(self):
        filename = None
        for x in LEASE_FILES:
            if os.path.isfile(x):
                filename = x
                break

        data = []
        for line in open(filename).readlines():
            if line.startswith("#"):
                continue
            data.append(line.strip())
        data = " ".join(data)
        items = data.split("lease")
        items = map(string.strip, items)
        for i in items:
            if len(i) < 2:
                continue
            ip = i.split()[0]
            desc = i.split("{")[1].split("}")[0]
            desc = map(string.strip, desc.split(";"))[:-1]
            rdesc = {}
            rdesc['ip'] = ip
            for x in desc:
                if "hardware ethernet" in x:
                    rdesc['mac'] = x.split()[2]
                if "client-hostname" in x:
                    rdesc['hostname'] = x.split()[1][1:-1]
                if "starts" in x:
                    rdesc['start'] = " ".join(x.split()[2:])
                if "ends" in x:
                    ts = " ".join(x.split()[2:])
                    rdesc['end'] = ts
                    tt = (ts[:4], ts[5:7], ts[8:10],
                          ts[11:13], ts[14:16],ts[17:19],
                          0, 0, 0)
                    end_ts = time.mktime(map(int, tt))
                    now = time.time()

                    rdesc['ended'] = (now - end_ts) > 0 and True or False
                
            if not rdesc.get('end', None):
                rdesc['ended'] = False
            self.leases[rdesc['ip']] = rdesc
            #self.leases.sort(cmp=lambda x,y: cmp(x['start'], y['start']))
            
    def llist(self, path):
        if not path:
            k = self.leases.keys()
            k.sort()
            return k
        else:
            if len(path) == 1:
                if self.leases.has_key(path[0]):
                    return self.leases[path[0]].keys()
            if len(path) == 2:
                return []

    def lhas(self, path):
        if len(path) == 1:
            if self.leases.has_key(path[0]):
                return True
        if len(path) == 2:
            if self.leases.has_key(path[0]):
                if self.leases[path[0]].has_key(path[1]):
                    return True
        
    def lget(self, path):
        if len(path) == 0:
            return self.leases
        if len(path) == 1:
            if self.leases.has_key(path[0]):
                return self.leases[path[0]]
        if len(path) == 2:
            if self.leases.has_key(path[0]):
                if self.leases[path[0]].has_key(path[1]):
                    return self.leases[path[0]][path[1]]
                
def init(**kw):
    filename = None
    for x in LEASE_FILES:
        if os.path.isfile(x):
            filename = x
            break

    if filename is not None:
        leases = DhcpLeases("leases")
    
        @mydeferred
        def update_all_data():
            lease.update_data()
            reactor.callLater(60, update_all_data)
            
        yield dict(
            tree = leases,
            root = '/dhcp'
            )
